# This is an example of how to convert all PDF files in a folder to DOCX files.
# 这是一个将文件夹中的所有 PDF 文件转换为 DOCX 文件的示例。

from pdfdeal import Doc2X

# gets API Key from environment variable DOC2X_APIKEY, or you can pass it as a string to the apikey parameter
# 从环境变量 DOC2X_APIKEY 获取 API Key, 或者您可以将其作为字符串传递给 apikey 参数

# client = Doc2X(apikey="Your API key",debug=True)
client = Doc2X(debug=True)

success, failed, flag = client.pdf2file(
    pdf_file="tests/pdf/test",
    output_path="./Output",
    output_format="docx",
)
print(success)
print(failed)
print(flag)



from bs4 import BeautifulSoup


def html_table_to_md(text: str) -> str:
    """Convert HTML tables to Markdown tables in the given text.

    Args:
        text (str): Text containing HTML tables

    Returns:
        str: Text with HTML tables converted to Markdown format
    """
    soup = BeautifulSoup(text, "html.parser")
    tables = soup.find_all("table")

    for table in tables:
        md_table = []
        max_cols = 0

        # Get all rows
        rows = table.find_all("tr")
        if not rows:
            continue

        for row in rows:
            cols = 0
            for cell in row.find_all(["td", "th"]):
                colspan = int(cell.get("colspan", 1))
                cols += colspan
            max_cols = max(max_cols, cols)

        for row in rows:
            row_data = []
            cells = row.find_all(["td", "th"])

            col_count = 0
            for cell in cells:
                content = cell.get_text().strip()
                colspan = int(cell.get("colspan", 1))
                for _ in range(colspan):
                    row_data.append(content)
                    col_count += 1

            while col_count < max_cols:
                row_data.append("")
                col_count += 1

            md_table.append("| " + " | ".join(row_data) + " |")

            if len(md_table) == 1:
                md_table.append("| " + " | ".join(["---"] * max_cols) + " |")

        # Replace the HTML table with markdown table
        md_table_str = "\n".join(md_table)
        table.replace_with(md_table_str)

    return str(soup)



import asyncio
from functools import wraps
import time
import sys
from concurrent.futures import ThreadPoolExecutor
import logging
from httpx import RemoteProtocolError, ConnectError, ConnectTimeout


async def code_check(code: str, uid: str = None, trace_id: str = None):
    if code in ["parse_page_limit_exceeded", "parse_concurrency_limit"]:
        raise RateLimit(trace_id=trace_id)
    if code in RequestError.ERROR_CODES:
        raise RequestError(code, uid=uid, trace_id=trace_id)
    if code == "unauthorized":
        raise ValueError("API key is unauthorized. (认证失败，请检测API key是否正确)")
    if code not in ["ok", "success"]:
        raise Exception(f"Unknown error code: {code}, UID: {uid}, Trace ID: {trace_id}")


class RateLimit(Exception):
    """
    Error when rate limit is reached.
    """

    def __init__(self, trace_id: str = None):
        self.trace_id = trace_id
        super().__init__()

    def __str__(self):
        trace_msg = f" (Trace ID: {self.trace_id})" if self.trace_id else ""
        return f"Rate limit reached. Please wait a moment and try again. (速率限制，请稍后重试){trace_msg}"


class RequestError(Exception):
    """
    Error when request is not successful, with known status code.
    """

    ERROR_CODES = {
        "parse_quota_limit": "可用的解析页数额度不足 (Insufficient parsing quota)",
        "parse_create_task_error": "创建任务失败 (Failed to create task)",
        "parse_file_too_large": "单个文件大小超过限制 (File size exceeds limit)",
        "parse_file_page_limit": "单个文件页数超过限制 (File page count exceeds limit)",
        "parse_file_lock": "文件解析失败 (File parsing failed)",
        "parse_pdf_invalid": "传入的文件不是有效的PDF文件 (File is not a valid PDF)",
        "parse_file_not_pdf": "传入的文件不是PDF文件 (File is not a PDF)",
        "parse_file_not_image": "传入的文件不在支持的图片文件范围内 (File is not a supported image type)",
        "internal_error": "内部错误 (Internal error)",
        "parse_concurrency_limit": "同时解析的PDF页数超出上限 (Concurrent PDF page parsing limit exceeded)",
        "parse_error": "解析失败 (Parsing failed)",
    }

    SOLUTIONS = {
        "parse_quota_limit": "当前可用的页数不足，请检查余额或联系负责人 (Insufficient parsing quota, check balance or contact support)",
        "parse_create_task_error": "短暂等待后重试, 如果还出现报错则请联系负责人 (Retry after a short wait, contact support if error persists)",
        "parse_file_too_large": "当前允许单个文件大小 <= 300MB(直接上传) | <= 1GB(通过OSS上传), 请拆分 pdf (File size must be <= 300MB (direct upload) | <= 1GB (OSS upload), please split the PDF)",
        "parse_file_page_limit": "当前允许单个文件页数 <= 1000页, 请拆分 pdf (File page count must be <= 1000 pages, please split the PDF)",
        "parse_file_lock": "为了防止反复解析, 暂时锁定一天,考虑PDF可能有兼容性问题, 重新打印后再尝试。仍然失败请反馈request_id给负责人 (Locked for a day to prevent repeated parsing. Consider reprinting the PDF if compatibility issues persist. Report request_id if it still fails)",
        "parse_pdf_invalid": "不是有效的PDF文件,考虑PDF可能有兼容性问题, 重新打印后再尝试。仍然失败请反馈request_id给负责人 (File is not a valid PDF. Consider reprinting the PDF if compatibility issues persist. Report request_id if it still fails)",
        "parse_file_not_pdf": "请解析后缀为.pdf的文件 (Please parse files with .pdf extension)",
        "parse_file_not_image": "目前只支持 jpg/png 图片文件的解析 (Currently only jpg/png image files are supported)",
        "internal_error": "请联系技术支持 (Please contact technical support)",
        "parse_concurrency_limit": "短暂等待后重试, 当前解析的PDF页数超出上限 (Retry after a short wait, concurrent PDF page parsing limit exceeded)",
        "parse_error": "短暂等待后重试, 如果还出现报错则请联系负责人 (Retry after a short wait, contact support if error persists)",
    }

    def __init__(self, error_code, uid: str = None, trace_id: str = None, message=None):
        self.error_code = error_code
        self.uid = uid
        self.trace_id = trace_id
        self.reason = self.ERROR_CODES.get(error_code, "未知错误 (Unknown error)")
        self.solution = self.SOLUTIONS.get(
            error_code, "请联系技术支持 (Please contact technical support)"
        )
        super().__init__(message or f"{self.error_code}: {self.reason}")

    def __str__(self):
        self.uid = (
            self.uid
            or "Failed to get uid! Please set DEBUG mode to check the failed file path."
        )
        return f"{self.error_code}: {self.reason}\nUID: {self.uid}\nTrace ID: {self.trace_id}\nYou can try to do:\n{self.solution}"


class FileError(Exception):
    """
    Error when file is not found or cannot be opened or othes.
    """

    pass


def async_retry(max_retries=2, backoff_factor=2, timeout=60):
    """
    Decorator to retry an async function when an exception is raised.
    `max_retries`: Maximum number of retries.
    `backoff_factor`: Factor to increase the wait time between retries.
    `timeout`: Timeout in seconds for each function call.
    """

    def decorator(func):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            for retries in range(max_retries + 1):
                try:
                    return await asyncio.wait_for(
                        func(*args, **kwargs), timeout=timeout
                    )
                except asyncio.TimeoutError:
                    if retries == max_retries:
                        logging.exception(
                            f"Function '{func.__name__}' timed out after {timeout} seconds"
                        )
                        raise
                    logging.warning(
                        f"Function '{func.__name__}' timed out, retrying..."
                    )
                except (
                    RateLimit,
                    FileError,
                    RequestError,
                    FileNotFoundError,
                    ValueError,
                ) as e:
                    logging.error(
                        f"Error in '{func.__name__}': {type(e).__name__} - {e}"
                    )
                    raise
                except (RemoteProtocolError, ConnectError, ConnectTimeout) as e:
                    if retries == max_retries:
                        logging.error(
                            f"Error in '{func.__name__}': {type(e).__name__} - {e}"
                        )
                        raise
                    wait_time = backoff_factor**retries
                    logging.warning(
                        f"{type(e).__name__}, this is most likely a network link issue, if this problem occurs frequently check your network environment (e.g. turn off your VPN, check your DNS seeting), will retry in {wait_time} seconds..."
                    )
                    await asyncio.sleep(wait_time)
                except Exception as e:
                    if isinstance(e, RequestError):
                        logging.error(str(e))
                        raise
                    elif isinstance(e, RateLimit):
                        raise
                    elif retries == max_retries:
                        logging.exception(
                            f"Error in '{func.__name__}': {type(e).__name__} - {e}"
                        )
                        raise
                    wait_time = backoff_factor**retries
                    logging.exception(
                        f"Exception in '{func.__name__}': {type(e).__name__} - {e}"
                    )
                    logging.warning(
                        f"Errors detected that may be resolved by retrying, will retrying in {wait_time} seconds..."
                    )
                    await asyncio.sleep(wait_time)

        return wrapper

    return decorator


def nomal_retry(max_retries=3, backoff_factor=2):
    """
    Decorator to retry an async function when an exception is raised.
    `max_retries`: Maximum number of retries.
    `backoff_factor`: Factor to increase the wait time between retries.
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            for retries in range(max_retries + 1):
                try:
                    return func(*args, **kwargs)
                except RateLimit:
                    raise
                except FileError as e:
                    logging.exception(f"FileError in '{func.__name__}': {e}")
                    raise
                except Exception as e:
                    if retries == max_retries:
                        logging.exception(
                            f"Max retries reached in '{func.__name__}': {e}"
                        )
                        raise
                    wait_time = backoff_factor**retries
                    logging.warning(
                        f"Retrying '{func.__name__}' in {wait_time} seconds. Error: {e}"
                    )
                    time.sleep(wait_time)

        return wrapper

    return decorator


def run_async(coro):
    """This function is used to run async function in sync way. As `asyncio.run` not work in jupyter notebook.

    Args:
        coro (_type_): The function to run.

    Returns:
        _type_: The result of the function.
    """
    if "IPython" in sys.modules:
        # Jupyter Notebook
        try:
            loop = asyncio.get_running_loop()
        except RuntimeError:
            loop = None

        if loop and loop.is_running():
            with ThreadPoolExecutor() as executor:
                future = executor.submit(asyncio.run, coro)
                return future.result()
    # Python
    return asyncio.run(coro)



import asyncio
import os
from .Doc2X.Exception import RateLimit, run_async
from .Doc2X.Types import RAG_OutputType
from .Doc2X.Types import OutputFormat_Legacy as OutputFormat
from .FileTools.dealpdfs import strore_pdf
from typing import Tuple
from .FileTools.file_tools import list_rename
import uuid
import logging

from .Doc2X.ConvertV1 import (
    refresh_key,
    get_limit,
    uuid2file,
    upload_pdf,
    upload_img,
    uuid_status,
    check_folder,
    process_status,
)


async def get_key(apikey: str) -> str:
    """Get apikey from environment variable or input

    Args:
        apikey (str): The apikey, or get from environment variable `DOC2X_APIKEY`

    Raises:
        ValueError: If no apikey found

    Returns:
        str: Your real apikey
    """
    if apikey is None:
        apikey = os.environ.get("DOC2X_APIKEY", "")
    if apikey == "":
        raise ValueError("No apikey found")
    if not apikey.startswith("sk-"):
        apikey = await refresh_key(apikey)
    return apikey


async def pdf2file_v1(
    apikey: str,
    pdf_path: str,
    output_path: str,
    output_format: str,
    ocr: bool,
    maxretry: int,
    rpm: int,
    convert: bool,
    translate: bool = False,
    language: str = "zh",
    model: str = "deepseek",
):
    """
    Convert pdf file to specified file,
    """
    # Upload the file and get uuid
    try:
        uuid = await upload_pdf(
            apikey=apikey,
            pdffile=pdf_path,
            ocr=ocr,
            translate=translate,
            language=language,
            model=model,
        )
    except RateLimit:
        while True:
            logging.warning("Reach the rate limit, wait for 25 seconds")
            await asyncio.sleep(25)
            try:
                uuid = await upload_pdf(
                    apikey=apikey,
                    pdffile=pdf_path,
                    ocr=ocr,
                    translate=translate,
                    language=language,
                    model=model,
                )
                break
            except RateLimit:
                continue
    # Wait for the process to finish
    while True:
        status_process, status_str, texts, other = await uuid_status(
            apikey=apikey, uuid=uuid, convert=convert, translate=translate
        )
        if status_process == 100 and status_str == "Success" and not translate:
            # If output_format is texts, return texts directly
            if output_format == "texts":
                return texts
            logging.info(
                f"{status_str} - Exporting document: {status_process}%    -- uuid: {uuid}"
            )
            break
        # If translate is True, return texts and other(texts location inside)
        elif status_process == 100 and status_str == "Translate success":
            final = {"texts": texts, "location": other}
            return final
        logging.info(f"{status_str}: {status_process}%    -- uuid: {uuid}")
        await asyncio.sleep(1)
    # Convert uuid to file
    return await uuid2file(
        apikey=apikey, uuid=uuid, output_path=output_path, output_format=output_format
    )


async def img2file_v1(
    apikey: str,
    img_path: str,
    output_path: str,
    output_format: str,
    formula: bool,
    img_correction: bool,
    maxretry: int,
    rpm: int,
    convert: bool,
) -> str:
    """
    Convert image file to specified file
    """
    try:
        uuid = await upload_img(
            apikey=apikey,
            imgfile=img_path,
            formula=formula,
            img_correction=img_correction,
        )
    except RateLimit:
        # Retry according to maxretry and current rpm
        while True:
            logging.warning("Reach the rate limit, wait for 25 seconds")
            await asyncio.sleep(25)
            try:
                uuid = await upload_img(
                    apikey=apikey,
                    imgfile=img_path,
                    formula=formula,
                    img_correction=img_correction,
                )
                break
            except RateLimit:
                continue
    # Wait for the process to finish
    while True:
        status_process, status_str, texts, other = await uuid_status(
            apikey=apikey, uuid=uuid, convert=convert
        )
        if status_process == 100 and status_str == "Success":
            # If output_format is texts, return texts directly
            if output_format == "texts":
                return texts
            logging.info(f"{status_str}: {status_process}%    -- uuid: {uuid}")
            break
        logging.info(f"{status_str}: {status_process}%    -- uuid: {uuid}")
        await asyncio.sleep(1)
    return await uuid2file(
        apikey=apikey, uuid=uuid, output_path=output_path, output_format=output_format
    )


class Doc2X:
    """Init the Doc2X class(V1)"""

    def __init__(
        self,
        apikey: str = None,
        rpm: int = None,
        thread: int = None,
    ) -> None:
        """Init the Doc2X class

        Args:
            apikey (str, optional): Your doc2x apikey. Defaults to read from environment variable `DOC2X_APIKEY`.
            rpm (int, optional): The rate of concurrent processing. Defaults will be auto set according to the apikey. Please use `thread` instead of `rpm`.
            thread (int, optional): The rate of concurrent processing. Defaults will be auto set according to the apikey.
        """
        self.apikey = run_async(get_key(apikey))
        if rpm is not None and thread is not None:
            raise ValueError(
                "Please use `rpm` or `thread`, not both. Suggest to use `thread`."
            )
        elif thread is not None:
            self.rpm = thread
        elif rpm is not None:
            import warnings

            warnings.warn(
                "The `rpm` parameter is deprecated and will be removed in the future. Please use the `thread` parameter instead.",
            )
            self.rpm = rpm
        else:
            if self.apikey.startswith("sk-"):
                self.rpm = 10
            else:
                self.rpm = 1
        self.maxretry = None

    async def pic2file_back(
        self,
        image_file: list,
        output_path: str = "./Output",
        output_format: str = "md_dollar",
        img_correction: bool = True,
        equation=False,
        convert: bool = False,
    ) -> str:
        """
        Convert image file to specified file, with rate/thread limit
        input refers to `pic2file` function
        """
        limit = asyncio.Semaphore(self.rpm)

        async def limited_img2file_v1(img):
            try:
                await limit.acquire()
                return await img2file_v1(
                    apikey=self.apikey,
                    img_path=img,
                    output_path=output_path,
                    output_format=output_format,
                    formula=equation,
                    img_correction=img_correction,
                    maxretry=self.maxretry,
                    rpm=self.rpm,
                    convert=convert,
                )
            except Exception as e:
                return f"Error {e}"
            finally:
                limit.release()

        task = [limited_img2file_v1(img) for img in image_file]
        completed_tasks = await asyncio.gather(*task)
        return await process_status(image_file, completed_tasks)

    def pic2file(
        self,
        image_file,
        output_path: str = "./Output",
        output_names: list = None,
        output_format: str = "md_dollar",
        img_correction: bool = True,
        equation: bool = False,
        convert: bool = False,
    ) -> Tuple[list, list, bool]:
        """Convert image file to specified file

        Args:
            image_file (str or list): Image file path, or a list of image file path
            output_path (str, optional): Output folder path. Defaults to "./Output".
            output_names (list, optional): Custom Output File Names, must be the same length as `image_file`. Defaults to None.
            output_format (str, optional): Output format, accept `texts`, `md`, `md_dollar`, `latex`, `docx`. Defaults to `md_dollar`.
            img_correction (bool, optional): The image correction. Defaults to True.
            equation (bool, optional): Whether the image is an equation. Defaults to False.
            convert (bool, optional): Whether to convert `[` to `$` and `[[` to `$$`. Defaults to False.

        Raises:
            ValueError: The length of files and output_names should be the same.

        Returns:
            tuple[list,list,str]:
            will return `list1`,`list2`,`bool`
                `list1`: list of successful files path, if some files are failed, its path will be empty string
                `list2`: list of failed files's error message and its original file path, id some files are successful, its error message will be empty string
                `bool`: True means that at least one file process failed
        """
        output_format = OutputFormat(output_format)
        if isinstance(output_format, OutputFormat):
            output_format = output_format.value

        if isinstance(image_file, str):
            image_file = [image_file]

        if output_names is not None:
            if len(image_file) != len(output_names):
                raise ValueError(
                    "The length of files and output_names should be the same."
                )

        success, failed, flag = run_async(
            self.pic2file_back(
                image_file,
                output_path,
                output_format,
                img_correction,
                equation,
                convert,
            )
        )

        logging.info(
            f"IMG Progress: {sum(1 for s in success if s != '')}/{len(image_file)} files successfully processed."
        )
        if flag:
            for failed_file in failed:
                if failed_file["error"] != "":
                    logging.error(
                        f"-----\nFailed deal with {failed_file['path']} with error:\n{failed_file['error']}\n-----"
                    )

        if output_names is not None:
            success = list_rename(success, output_names)

        return success, failed, flag

    async def pdf2file_back(
        self,
        pdf_file: list,
        output_path: str = "./Output",
        output_format: str = "md_dollar",
        ocr: bool = True,
        convert: bool = False,
        translate: bool = False,
        language: str = "zh",
        model: str = "deepseek",
    ) -> str:
        """
        Convert pdf file to specified file, with rate/thread limit, async version
        input refers to `pdf2file` function
        """
        limit = asyncio.Semaphore(self.rpm)

        async def limited_pdf2file_v1(pdf):
            try:
                await limit.acquire()
                return await pdf2file_v1(
                    apikey=self.apikey,
                    pdf_path=pdf,
                    output_path=output_path,
                    output_format=output_format,
                    ocr=ocr,
                    maxretry=self.maxretry,
                    rpm=self.rpm,
                    convert=convert,
                    translate=translate,
                    language=language,
                    model=model,
                )
            except Exception as e:
                return f"Error {e}"
            finally:
                limit.release()

        tasks = [limited_pdf2file_v1(pdf) for pdf in pdf_file]
        completed_tasks = await asyncio.gather(*tasks)
        return await process_status(pdf_file, completed_tasks)

    def pdf2file(
        self,
        pdf_file,
        output_path: str = "./Output",
        output_names: list = None,
        output_format: str = "md_dollar",
        ocr: bool = True,
        convert: bool = False,
    ) -> Tuple[list, list, bool]:
        """Convert pdf file to specified file

        Args:
            pdf_file (str or list): pdf file path, or a list of pdf file path
            output_path (str, optional): output folder path. Defaults to "./Output".
            output_names (list, optional): Custom Output File Names, must be the same length as `pdf_file`. Defaults to None.
            output_format (str, optional): output format, accept `texts`, `md`, `md_dollar`, `latex`, `docx`. Defaults to `md_dollar`.
            ocr (bool, optional): whether to use OCR. Defaults to True.
            convert (bool, optional): whether to convert `[` to `$` and `[[` to `$$`. Defaults to False.

        Raises:
            ValueError: The length of files and output_names should be the same.

        Returns:
            tuple[list,list,str]:
            will return `list1`,`list2`,`bool`
                `list1`: list of successful files path, if some files are failed, its path will be empty string
                `list2`: list of failed files's error message and its original file path, id some files are successful, its error message will be empty string
                `bool`: True means that at least one file process failed
        """
        output_format = OutputFormat(output_format)
        if isinstance(output_format, OutputFormat):
            output_format = output_format.value

        if isinstance(pdf_file, str):
            pdf_file = [pdf_file]

        if output_names is not None:
            if len(pdf_file) != len(output_names):
                raise ValueError(
                    "The length of files and output_names should be the same."
                )

        success, failed, flag = run_async(
            self.pdf2file_back(
                pdf_file, output_path, output_format, ocr, convert, False
            )
        )
        logging.info(
            f"PDF Progress: {sum(1 for s in success if s != '')}/{len(pdf_file)} files successfully processed."
        )
        if flag:
            for failed_file in failed:
                if failed_file["error"] != "":
                    logging.error(
                        f"-----\nFailed deal with {failed_file['path']} with error:\n{failed_file['error']}\n-----"
                    )

        if output_names is not None:
            success = list_rename(success, output_names)

        return success, failed, flag

    def get_limit(self) -> int:
        """Get the rate limit of the apikey

        Returns:
            int: The rate limit of the apikey
        """
        return run_async(get_limit(self.apikey))

    async def pdfdeal_back(
        self,
        input: str,
        output: str,
        path: str,
        convert: bool,
        limit: asyncio.Semaphore,
    ):
        """
        Convert pdf files into recognisable pdfs, significantly improving their effectiveness in RAG systems
        async version function
        """
        try:
            await limit.acquire()
            texts = await pdf2file_v1(
                apikey=self.apikey,
                pdf_path=input,
                output_path=output,
                output_format="texts",
                ocr=True,
                maxretry=self.maxretry,
                rpm=self.rpm,
                convert=convert,
                translate=False,
            )
            await check_folder(path)
            file_name = os.path.basename(input).replace(
                ".pdf", f"_{uuid.uuid4()}.{output}"
            )
            output_path = os.path.join(path, file_name)
            if output == "pdf":
                strore_pdf(output_path, texts)
            elif output == "texts":
                return texts, "", True
            else:
                md_text = "\n".join(texts) + "\n"
                with open(output_path, "w", encoding="utf-8") as f:
                    f.write(md_text)
            return output_path, "", True
        except Exception as e:
            return input, e, False
        finally:
            limit.release()

    async def pdfdeals(
        self,
        pdf_files: list,
        output_path: str = "./Output",
        output_format: str = "pdf",
        convert: bool = True,
    ) -> list:
        """
        Convert pdf files into recognisable pdfs, significantly improving their effectiveness in RAG systems
        async version function, input refers to `pdfdeal` function
        """
        limit = asyncio.Semaphore(self.rpm)
        tasks = [
            self.pdfdeal_back(pdf_file, output_format, output_path, convert, limit)
            for pdf_file in pdf_files
        ]
        completed_tasks = await asyncio.gather(*tasks)
        success_file = []
        error_file = []
        error_flag = False
        for temp in completed_tasks:
            path = temp[0]
            error = temp[1]
            flag = temp[2]
            if flag:
                success_file.append(path)
                error_file.append({"error": "", "path": ""})
            else:
                success_file.append("")
                error_file.append({"error": error, "path": path})
                error_flag = True
        return success_file, error_file, error_flag

    def pdfdeal(
        self,
        pdf_file,
        output_format: str = "pdf",
        output_names: list = None,
        output_path: str = "./Output",
        convert: bool = True,
    ) -> Tuple[list, list, bool]:
        """Deal with pdf file, convert it to specified format for RAG system

        Args:
            pdf_file (str or list): input file path, or a list of input file path
            output_format (str, optional): output format, accept 'pdf', 'md' or 'texts'. Defaults to "pdf".
            output_names (list, optional): Custom Output File Names, must be the same length as `image_file`. Defaults to None.
            output_path (str, optional): output path. Defaults to "./Output".
            convert (bool, optional): Whether to convert "[" to "$" and "[[" to "$$". Defaults to True.

        Returns:
            tuple[list,list,str]:
            will return `list1`,`list2`,`bool`
                `list1`: list of successful files path, if some files are failed, its path will be empty string
                `list2`: list of failed files's error message and its original file path, id some files are successful, its error message will be empty string
                `bool`: True means that at least one file process failed
        """
        output_format = RAG_OutputType(output_format)
        if isinstance(output_format, RAG_OutputType):
            output_format = output_format.value

        if isinstance(pdf_file, str):
            pdf_file = [pdf_file]

        success, failed, flag = run_async(
            self.pdfdeals(pdf_file, output_path, output_format, convert)
        )
        logging.info(
            f"PDFDEAL Progress: {sum(1 for s in success if s != '')}/{len(pdf_file)} files successfully processed."
        )
        if flag:
            for failed_file in failed:
                if failed_file["error"] != "":
                    logging.error(
                        f"-----\nFailed deal with {failed_file['path']} with error:\n{failed_file['error']}\n-----"
                    )

        if output_names is not None:
            success = list_rename(success, output_names)

        return success, failed, flag



import argparse
import os
from pdfdeal import Doc2X


def main():
    parser = argparse.ArgumentParser(
        description="Using doc2x to deal with pictures or pdfs"
    )
    parser.add_argument("filename", help="PDF file/folder", nargs="?")
    parser.add_argument(
        "-y",
        help="Will skip any scenarios that require a second input from the user.",
        required=False,
        action="store_true",
    )
    parser.add_argument(
        "-k",
        "--api_key",
        help="The API key of Doc2X, if not set, will use the global setting",
        required=False,
    )
    parser.add_argument(
        "--thread",
        help="The thread limit of request, DO NOT set if you don't know",
        required=False,
    )
    parser.add_argument(
        "--max_pages",
        help="The maximum number of pages to process at same time, default is 1000, DO NOT set if you don't know",
        required=False,
    )
    parser.add_argument(
        "-o",
        "--output",
        help="The output folder of the result, if not set, will set to './Output'",
        required=False,
    )
    parser.add_argument(
        "-f",
        "--format",
        help="The output format of the result, accept md、md_dollar、tex、docx, default is md_dollar",
        required=False,
        choices=["md", "md_dollar", "tex", "docx"],
    )
    parser.add_argument(
        "--graphrag",
        help="Change md document to txt form, used for output is converted to the txt form accepted by graphRAG. The output format needs to be md or md_dollar at this time",
        required=False,
        action="store_true",
    )
    parser.add_argument(
        "--unzip",
        help="Unzip the output file, only works for the output is zip file",
        required=False,
        action="store_true",
    )
    # Only if need user input, will ask language
    args = parser.parse_args()

    if args.api_key is None:
        api_key = os.getenv("DOC2X_APIKEY")
        if api_key is None and not args.y:
            api_key = input("Please input the API key of Doc2X: ")
    else:
        api_key = str(args.api_key)

    if args.thread is not None:
        thread = int(args.thread)
    else:
        thread = 5

    if args.max_pages is not None:
        max_pages = int(args.max_pages)
    else:
        max_pages = 1000

    if args.filename is None:
        if args.y:
            filename = "./"
        else:
            filename = input("Please input the file/folder path: ")
    else:
        filename = args.filename

    output = args.output if args.output else "./Output"

    format = args.format if args.format else "md_dollar"

    Client = Doc2X(apikey=api_key, thread=thread, max_pages=max_pages, debug=True)

    if args.graphrag:
        assert format in [
            "md",
            "md_dollar",
        ], "The output format needs to be md or md_dollar at this time"

    success, fail, flag = Client.pdf2file(
        pdf_file=filename,
        output_path=output,
        output_format=format,
    )

    for file in success:
        if file != "":
            file = os.path.abspath(file)
            print(f"Save to: {file}")
    if flag:
        print("Some files failed to process, please check the error message.")
        print("Try to save the failed file path and reasons to fail.txt")
        try:
            with open("fail.txt", "w") as f:
                for file in fail:
                    if file["path"] != "":
                        f.write(str(file))
        except Exception as e:
            print(f"Failed to save the failed files to fail.txt: {e}")
            print("The failed files are:")
            for file in fail:
                print(file)

    if args.graphrag or args.unzip:
        from pdfdeal.FileTools.file_tools import unzip

        for file in success:
            if file != "" and file.endswith(".zip"):
                file = os.path.abspath(file)
                try:
                    unzip(zip_path=file, rename=True)
                except Exception as e:
                    print(f"Failed to unzip the file: {file}, error: {e}")
        output_folder = os.path.abspath(output)
        if args.graphrag:
            for root, dirs, files in os.walk(output_folder):
                for file in files:
                    if file.endswith(".md"):
                        file_path = os.path.join(root, file)
                        new_file_path = file_path[:-3] + ".txt"
                        os.rename(file_path, new_file_path)
        print(f"Unzip and rename the files in {output_folder} successfully.")

    print("==Have a nice day!===")


if __name__ == "__main__":
    main()



"""The connect funcs of the project"""

BUILD_IN_CONNECT = ["local_folder"]


def load_build_in_connect(connect: str):
    """
    Load the build-in connect engine
    """
    connect_mapping = {"local_folder": local_folder_connect}
    return connect_mapping.get(connect)


def local_folder_connect():
    """Connect to a local folder"""
    from .Connect.local import connect, config, get

    return connect, config, get



import httpx
import logging


class PicGO_Settings:
    def __init__(self, endpoint="http://127.0.0.1:36677"):
        """Initialize the PicGO client.

        Args:
            endpoint (str): The endpoint for PicGO API. Defaults to "http://127.0.0.1:36677".
        """
        self.endpoint = endpoint

    def upload_file(self, local_file_path, remote_file_path=None):
        """Upload a file to PicGO.

        Args:
            local_file_path (str): The path of the local file to upload.
            remote_file_path (str): Not used in PicGO, kept for interface consistency.

        Returns:
            tuple: A tuple containing the URL of the uploaded file and a boolean indicating whether the upload was successful.
        """
        try:
            data = {"list": [local_file_path]}
            response = httpx.post(
                f"{self.endpoint}/upload",
                json=data,
                headers={"Content-Type": "application/json"},
            )
            result = response.json()

            if result.get("success"):
                return result["result"][0], True
            else:
                logging.error(
                    f"Failed to upload file: {local_file_path}, PicGO response: {result}"
                )
                return result, False

        except Exception as e:
            logging.exception(f"Error uploading file: {local_file_path}, {str(e)}")
            return str(e), False


def PicGO(endpoint="http://127.0.0.1:36677") -> callable:
    """Initialize the PicGO client and return a callable function to upload files.

    Args:
        endpoint (str): The endpoint for PicGO API. Defaults to "http://127.0.0.1:36677".

    Returns:
        callable: The upload_file method of the PicGO client.
    """
    picgo_uploader = PicGO_Settings(endpoint=endpoint)
    return picgo_uploader.upload_file



import os
from minio import Minio, S3Error
import logging
from urllib.parse import urlparse
import json


class MINIO:
    def __init__(self, minio_address, minio_admin, minio_password, bucket_name):
        # 通过ip 账号 密码 连接minio server
        # 根据地址自动判断是否使用安全连接
        parsed_url = urlparse(minio_address)
        secure = parsed_url.scheme == "https"
        self.minioClient = Minio(
            endpoint=parsed_url.netloc,
            access_key=minio_admin,
            secret_key=minio_password,
            secure=secure,
        )
        self.bucket_name = bucket_name
        self.minio_address = minio_address

    def upload_file(self, local_file_path, remote_file_path):
        """Upload a file

        Args:
            local_file_path (str): The path of the local file to upload.
            remote_file_path (str): The path of the remote file to upload to.

        Returns:
            tuple: A tuple containing the URL of the uploaded file and a boolean indicating whether the upload was successful.
        """
        # 检查桶是否存在，不存在则新建并设置为公开只读
        check_bucket = self.minioClient.bucket_exists(self.bucket_name)
        if not check_bucket:
            self.minioClient.make_bucket(self.bucket_name)
            # 设置桶策略为公开只读
            policy = {
                "Version": "2012-10-17",
                "Statement": [
                    {
                        "Effect": "Allow",
                        "Principal": {"AWS": "*"},
                        "Action": ["s3:GetBucketLocation", "s3:ListBucket"],
                        "Resource": f"arn:aws:s3:::{self.bucket_name}",
                    },
                    {
                        "Effect": "Allow",
                        "Principal": {"AWS": "*"},
                        "Action": "s3:GetObject",
                        "Resource": f"arn:aws:s3:::{self.bucket_name}/*",
                    },
                ],
            }
            self.minioClient.set_bucket_policy(self.bucket_name, json.dumps(policy))
        try:
            path, file_name = os.path.split(local_file_path)
            self.minioClient.fput_object(
                bucket_name=self.bucket_name,
                object_name=file_name,
                file_path=local_file_path,
            )
            parsed_url = urlparse(self.minio_address)
            scheme = parsed_url.scheme or "http"
            remote_file_path = (
                f"{scheme}://{parsed_url.netloc}/{self.bucket_name}/{file_name}"
            )
            return (remote_file_path, True)
        except FileNotFoundError as err:
            logging.exception(f"Error to upload the file: {local_file_path}, {err}")
            return err, False
        except S3Error as err:
            logging.exception(f"Error to upload the file: {local_file_path}, {err}")
            return err, False


def Min(minio_address, minio_admin, minio_password, bucket_name) -> callable:
    Min_uploader = MINIO(
        minio_address=minio_address,
        minio_admin=minio_admin,
        minio_password=minio_password,
        bucket_name=bucket_name,
    )
    return Min_uploader.upload_file



import asyncio
import os
from typing import Tuple, List
import logging
from .Doc2X.ConvertV2 import (
    upload_pdf,
    uid_status,
    convert_parse,
    get_convert_result,
    download_file,
)
from .Doc2X.Types import OutputFormat
from .Doc2X.Pages import get_pdf_page_count
from .Doc2X.Exception import RequestError, RateLimit, run_async
from .FileTools.file_tools import get_files
import time

logger = logging.getLogger(name="pdfdeal.doc2x")


async def parse_pdf(
    apikey: str,
    pdf_path: str,
    maxretry: int,
    wait_time: int,
    max_time: int,
    convert: bool,
    oss_choose: str = "auto",
) -> Tuple[str, List[str], List[dict]]:
    """Parse PDF file and return uid and extracted text"""

    async def task_limit_lock():
        global full_speed
        if full_speed:
            global limit_lock, get_max_limit, max_threads, thread_min
            nonlocal thread_lock
            if not get_max_limit and not thread_lock:
                get_max_limit = True
                thread_lock = True
                async with limit_lock:
                    max_threads = max(thread_min, max_threads - 1)
            else:
                if not thread_lock:
                    async with limit_lock:
                        max_threads = max(thread_min, max_threads - 1)
                    thread_lock = True

    thread_lock = False
    for attempt in range(maxretry):
        try:
            logger.info(f"Uploading {pdf_path}...")
            uid = await upload_pdf(apikey, pdf_path, oss_choose)
            logger.info(f"Uploading successful for {pdf_path} with uid {uid}")

            for _ in range(max_time // 3):
                try:
                    progress, status, texts, locations = await uid_status(
                        apikey, uid, convert
                    )
                    if status == "Success":
                        logger.info(f"Parsing successful for {pdf_path} with uid {uid}")
                        return uid, texts, locations
                    elif status == "Processing file":
                        logger.info(f"Processing {uid} : {progress}%")
                        await asyncio.sleep(3)
                    else:
                        raise RequestError(
                            f"Unexpected status: {status} with uid: {uid}"
                        )
                except RateLimit:
                    logger.warning(
                        "Rate limit reached during status check, retrying from upload..."
                    )
                    await task_limit_lock()
                    await asyncio.sleep(wait_time)
                    break
            else:
                raise RequestError(f"Max time reached for uid_status with uid: {uid}")
        except RateLimit:
            if attempt < maxretry - 1:
                await task_limit_lock()
                logger.warning("Rate limit reached during upload, retrying...")
                await asyncio.sleep(wait_time)
            else:
                raise RequestError(
                    "Max retry reached for parse_pdf, this may be a rate limit issue, try to reduce the number of threads."
                )

    raise RequestError("Failed to parse PDF after maximum retries")


async def convert_to_format(
    apikey: str,
    uid: str,
    output_format: str,
    output_path: str,
    output_name: str,
    max_time: int,
) -> str:
    """Convert parsed PDF to specified format"""

    logger.info(f"Converting {uid} to {output_format}...")
    status, url = await convert_parse(apikey, uid, output_format)
    for _ in range(max_time // 3):
        if status == "Success":
            logger.info(f"Downloading {uid} {output_format} file to {output_path}...")
            return await download_file(
                url=url,
                file_type=output_format,
                target_folder=output_path,
                target_filename=output_name or uid,
            )
        elif status == "Processing":
            logger.info(f"Converting {uid} {output_format} file...")
            await asyncio.sleep(3)
            status, url = await get_convert_result(apikey, uid)
        else:
            raise RequestError(f"Unexpected status: {status} with uid: {uid}")
    raise RequestError(f"Max time reached for get_convert_result with uid: {uid}")


class Doc2X:
    def __init__(
        self,
        apikey: str = None,
        thread: int = 5,
        max_pages: int = 1000,
        retry_time: int = 5,
        max_time: int = 300,
        debug: bool = False,
        full_speed: bool = False,
    ) -> None:
        """
        Initialize a Doc2X client.

        Args:
            apikey (str, optional): The API key for Doc2X. If not provided, it will try to get from environment variable 'DOC2X_APIKEY'.
            thread (int, optional): The maximum number of concurrent threads at same time. Defaults to 5.
            max_pages (int, optional): The maximum number of pages to process at same time. Defaults to 1000.
            retry_time (int, optional): The number of retry attempts. Defaults to 5.
            max_time (int, optional): The maximum time (in seconds) to wait for a response. Defaults to 300.
            debug (bool, optional): Whether to enable debug logging. Defaults to False.
            full_speed (bool, optional): **Experimental function**. Whether to enable automatic sniffing of the concurrency limit. Defaults to False.

        Raises:
            ValueError: If no API key is found.

        Note:
            If debug is set to True, it will set the logging level of 'pdfdeal' logger to DEBUG.
        """
        self.apikey = apikey or os.environ.get("DOC2X_APIKEY", "")
        if not self.apikey:
            raise ValueError("No apikey found")
        self.retry_time = retry_time
        self.max_time = max_time
        self.thread = thread
        self.max_pages = max_pages
        self.request_interval = 0.1
        self.full_speed = full_speed

        handler = logging.StreamHandler()
        formatter = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        )
        handler.setFormatter(formatter)
        logger.handlers.clear()
        logger.addHandler(handler)
        logger.propagate = False
        if debug:
            logging.getLogger("pdfdeal").setLevel(logging.DEBUG)
        self.debug = debug

    async def pdf2file_back(
        self,
        pdf_file,
        output_names: List[str] = None,
        output_path: str = "./Output",
        output_format: str = "md_dollar",
        convert: bool = False,
        oss_choose: str = "auto",
    ) -> Tuple[List[str], List[dict], bool]:
        if isinstance(pdf_file, str):
            if os.path.isdir(pdf_file):
                pdf_file, output_names = get_files(
                    # 实际上由于文件后缀名是在下载阶段才确定的，所以这里的后缀名是无效的，只是为了向上兼容
                    path=pdf_file,
                    mode="pdf",
                    out="md_dollar",
                )
            else:
                pdf_file = [pdf_file]
                if output_names is None:
                    output_names = [os.path.basename(pdf_file[0])]

        output_names = output_names or [None] * len(pdf_file)
        if len(pdf_file) != len(output_names):
            raise ValueError("The length of files and output_names should be the same.")

        output_formats = []
        if isinstance(output_format, str):
            if "," in output_format:
                output_formats = [fmt.strip() for fmt in output_format.split(",")]
                logger.warning(
                    "You are using multiple output formats, this will increase many time, please be patient.(您正在使用多个输出格式，这将增加很多时间，请耐心等待。)"
                )
            else:
                output_formats = [output_format]
        else:
            raise ValueError("Invalid output format, should be a string.")

        for fmt in output_formats:
            fmt = OutputFormat(fmt)
            if isinstance(fmt, OutputFormat):
                fmt = fmt.value

        # Track total pages and last request time
        total_pages = 0
        last_request_time = 0
        page_lock = asyncio.Lock()
        parse_tasks = set()
        convert_tasks = set()
        results = [None] * len(pdf_file)
        parse_results = [None] * len(pdf_file)
        global limit_lock, get_max_limit, max_threads, full_speed, thread_min
        thread_min = self.thread
        full_speed = self.full_speed
        limit_lock = asyncio.Lock()
        get_max_limit = False
        max_threads = self.thread

        if full_speed:
            self.max_time = 600
            self.retry_time = 10
            self.request_interval = 0.01

        async def process_file(index, pdf, name):
            try:
                page_count = get_pdf_page_count(pdf)
            except RequestError as e:
                results[index] = ("", str(e), True)
                logger.warning(f"Skiping {pdf}: {str(e)}")
                return
            except Exception as e:
                logger.warning(f"Failed to get page count for {pdf}: {str(e)}")
                page_count = self.max_pages - 1  #! Assume the worst case
            if page_count > self.max_pages:
                logger.warning(f"File {pdf} has too many pages, skipping.")
                results[index] = ("", "File has too many pages", True)
                return

            nonlocal total_pages, last_request_time

            try:
                # Check if we can start new task
                while True:
                    async with page_lock:
                        if total_pages + page_count <= self.max_pages:
                            current_time = time.time()
                            if current_time - last_request_time < self.request_interval:
                                await asyncio.sleep(
                                    self.request_interval
                                    - (current_time - last_request_time)
                                )

                            total_pages += page_count
                            last_request_time = time.time()
                            break
                    await asyncio.sleep(0.1)

                # Process the file
                try:
                    uid, texts, locations = await parse_pdf(
                        apikey=self.apikey,
                        pdf_path=pdf,
                        maxretry=self.retry_time,
                        wait_time=5,
                        max_time=self.max_time,
                        convert=convert,
                        oss_choose=oss_choose,
                    )
                    parse_results[index] = (uid, texts, locations)
                    # Create convert task as soon as parse is complete
                    task = asyncio.create_task(convert_file(index, name))
                    convert_tasks.add(task)

                except asyncio.TimeoutError:
                    results[index] = (
                        "",
                        "Operation timed out, this may be a rate limit issue or network issue, try to reduce the number of threads.",
                        True,
                    )
                except Exception as e:
                    results[index] = ("", str(e), True)
            finally:
                async with page_lock:
                    total_pages -= page_count

        async def convert_file(index, name):
            if parse_results[index] is None:
                return
            uid, texts, locations = parse_results[index]
            all_results = []
            all_errors = []

            for name_index, fmt in enumerate(output_formats):
                if isinstance(name, list):
                    try:
                        name_fmt = name[name_index]
                    except IndexError:
                        name_fmt = name[-1]
                else:
                    name_fmt = name
                try:
                    if fmt in ["md", "md_dollar", "tex", "docx"]:
                        nonlocal last_request_time
                        # Wait for request interval
                        current_time = time.time()
                        if current_time - last_request_time < self.request_interval:
                            await asyncio.sleep(
                                self.request_interval
                                - (current_time - last_request_time)
                            )

                        async with page_lock:
                            last_request_time = time.time()

                        result = await convert_to_format(
                            apikey=self.apikey,
                            uid=uid,
                            output_format=fmt,
                            output_path=output_path,
                            output_name=name_fmt,
                            max_time=self.max_time,
                        )
                        all_results.append(result)
                        all_errors.append("")
                        # Wait 5 seconds between formats
                        if fmt != output_formats[-1]:
                            logger.info(
                                f"Due to the rate limit, waiting 5 seconds before converting {pdf_file[index]} to the{fmt} format."
                            )
                            await asyncio.sleep(5)
                    else:
                        if fmt == "texts":
                            result = texts
                        elif fmt == "text":
                            result = "\n".join(texts)
                        elif fmt == "detailed":
                            result = [
                                {"text": text, "location": loc}
                                for text, loc in zip(texts, locations)
                            ]
                        all_results.append(result)
                        all_errors.append("")

                except asyncio.TimeoutError:
                    all_results.append("")
                    all_errors.append(
                        f"Operation timed out while converting to {fmt}, this may be a rate limit issue or network issue, try to reduce the number of threads."
                    )
                except Exception as e:
                    all_results.append("")
                    error_message = str(e) if str(e) else type(e).__name__
                    all_errors.append(
                        f"Error while converting to {fmt}: {error_message}"
                    )

            results[index] = (
                (all_results[0], all_errors[0], not all_results[0])
                if len(all_results) == 1 and len(all_errors) == 1
                else (
                    all_results,
                    all_errors,
                    any(not result for result in all_results),
                )
            )

        # Create and run parse tasks with controlled concurrency
        for i, (pdf, name) in enumerate(zip(pdf_file, output_names)):
            while len(parse_tasks) >= max_threads:
                done, parse_tasks = await asyncio.wait(
                    parse_tasks, return_when=asyncio.FIRST_COMPLETED
                )
                if full_speed:
                    async with limit_lock:
                        if not get_max_limit:
                            max_threads = max_threads + 1
            task = asyncio.create_task(process_file(i, pdf, name))
            parse_tasks.add(task)

        # Wait for remaining parse tasks
        if parse_tasks:
            await asyncio.wait(parse_tasks)

        # Wait for remaining convert tasks
        if convert_tasks:
            await asyncio.wait(convert_tasks)
        else:
            logger.warning("No successful parse tasks, skipping conversion.")

        if full_speed:
            logger.info(f"Convert tasks done with {max_threads} threads.")
        success_files = []
        for r in results:
            success_files.append(r[0])

        failed_files = []
        for r, pdf in zip(results, pdf_file):
            if r and r[2]:
                failed_files.append({"error": r[1], "path": pdf})
            else:
                failed_files.append({"error": r[1], "path": ""})
        has_error = any(r[2] for r in results if r)
        if has_error:
            logger.error(
                f"Failed to convert {sum(1 for r in results if r and r[2])} file(s), please enable DEBUG mode to check or read the output variable."
            )
            if self.debug:
                if has_error:
                    print("=====================")
                    for fail in failed_files:
                        if isinstance(fail["error"], list):
                            for e in fail["error"]:
                                if e != "":
                                    print(
                                        f"Failed to convert {fail['path']}: {e}\n====================="
                                    )
                        else:
                            if fail["error"] != "":
                                print(
                                    f"Failed to convert {fail['path']}: {fail['error']}\n====================="
                                )

        logger.info(
            f"Successfully converted {sum(1 for r in results if r and not r[2])} file(s) and failed to convert {sum(1 for r in results if r and r[2])} file(s)."
        )
        return success_files, failed_files, has_error

    def pdf2file(
        self,
        pdf_file,
        output_names: List[str] = None,
        output_path: str = "./Output",
        output_format: str = "md_dollar",
        convert: bool = False,
        oss_choose: str = "always",
        ocr: bool = False,
    ) -> Tuple[List[str], List[dict], bool]:
        """Convert PDF files to the specified format.

        Args:
            pdf_file (str | List[str]): Path to a single PDF file or a list of PDF file paths.
            output_names (List[str], optional): List of output file names. Defaults to None.
            output_path (str, optional): Directory path for output files. Defaults to "./Output".
            output_format (str, optional): Desired output format. Defaults to `md_dollar`. Supported formats include:`md_dollar`|`md`|`tex`|`docx`, will return the path of files, support output variable: `text`|`texts`|`detailed`(it means `string in md format`, `list of strings split by page`, `list of strings split by page (including detailed page information)`)
            convert (bool, optional): Whether to convert "[" and "[[" to "$" and "$$", only valid if `output_format` is a variable format(`txt`|`txts`|`detailed`). Defaults to False.
            oss_choose (str, optional): Now can upload files directly through API or through OSS link given by API. Acceptable values: `auto`, `always`, `never` (it means `Only >=100MB files will be uploaded to OSS`, `All files will be uploaded to OSS`, `All files will be uploaded directly`). Defaults to "always".
            ocr (bool, optional): This option is deprecated and will not be used.

        Returns:
            Tuple[List[str], List[dict], bool]: A tuple containing:
                1. A list of successfully converted file paths or content.
                2. A list of dictionaries containing error information for failed conversions.
                3. A boolean indicating whether any errors occurred during the conversion process.

        Raises:
            Any exceptions raised by pdf2file_back or run_async.

        Note:
            This method provides a convenient synchronous interface for the asynchronous
            PDF conversion functionality. It handles all the necessary setup for running
            the asynchronous code in a synchronous context.
        """
        if ocr:
            import warnings

            warnings.warn(
                "The 'ocr' option is deprecated and will not be used.",
                DeprecationWarning,
                stacklevel=2,
            )

        return run_async(
            self.pdf2file_back(
                pdf_file=pdf_file,
                output_names=output_names,
                output_path=output_path,
                output_format=output_format,
                convert=convert,
                oss_choose=oss_choose,
            )
        )



import boto3
from botocore.exceptions import NoCredentialsError, ClientError
import logging


class S3_Settings:
    def __init__(
        self,
        S3_ACCESS_KEY_ID,
        S3_ACCESS_KEY_SECRET,
        Endpoint,
        Bucket,
        Customized_Domain,
    ):
        """Initialize the S3 client.

        Args:
            S3_ACCESS_KEY_ID (str): The access key ID for S3.
            S3_ACCESS_KEY_SECRET (str): The access key secret for S3.
            Endpoint (str): The endpoint for S3.
            Bucket (str): The name of the bucket to upload to.
            Customized_Domain (str): The customized domain for the S3 bucket. **Note the URL of the imgs will be `{Customized_Domain}/{remote_file_path}`**
        """
        self.s3_client = boto3.client(
            "s3",
            endpoint_url=Endpoint,
            aws_access_key_id=S3_ACCESS_KEY_ID,
            aws_secret_access_key=S3_ACCESS_KEY_SECRET,
        )
        self.bucket = Bucket
        self.customized_domain = Customized_Domain

    def upload_file(self, local_file_path, remote_file_path):
        """Upload a file to S3.

        Args:
            local_file_path (str): The path of the local file to upload.
            remote_file_path (str): The path of the remote file to upload to.

        Returns:
            tuple: A tuple containing the URL of the uploaded file and a boolean indicating whether the upload was successful.
        """
        try:
            self.s3_client.upload_file(
                local_file_path,
                self.bucket,
                remote_file_path,
                ExtraArgs={"ACL": "public-read"},
            )
            # * Default think the bucket is public read
            return (
                f"{self.customized_domain}/{remote_file_path}",
                True,
            )
        except (NoCredentialsError, ClientError) as e:
            logging.exception(f"Error to upload the file: {local_file_path}, {e}")
            return e, False


def S3(
    S3_ACCESS_KEY_ID, S3_ACCESS_KEY_SECRET, Endpoint, Bucket, Customized_Domain
) -> callable:
    """Initialize the S3 client and return a callable function to upload files.

    Args:
        S3_ACCESS_KEY_ID (str): The access key ID for S3.
        S3_ACCESS_KEY_SECRET (str): The access key secret for S3.
        Endpoint (str): The endpoint for S3.
        Bucket (str): The name of the bucket to upload to.
        Customized_Domain (str): The customized domain for the S3 bucket. **Note the URL of the imgs will be `{Customized_Domain}/{remote_file_path}`**

    Returns:
        callable: The upload_file method of the S3 client.
    """
    s3_uploader = S3_Settings(
        S3_ACCESS_KEY_ID=S3_ACCESS_KEY_ID,
        S3_ACCESS_KEY_SECRET=S3_ACCESS_KEY_SECRET,
        Endpoint=Endpoint,
        Bucket=Bucket,
        Customized_Domain=Customized_Domain,
    )
    return s3_uploader.upload_file



import os
import json


def set_global_setting():
    """Set the global setting for pdfdeal"""
    from .config import init_config

    base_folder = os.path.expanduser("~/pdfdeal")
    global_setting_file = os.path.join(base_folder, "global_setting.json")
    config = init_config()
    with open(global_setting_file, "w") as file:
        json.dump(config, file)


def change_one_global_setting(key: str, value: str):
    """Change one global setting or add a new one if not exist"""
    try:
        global_setting = get_global_setting()
    except FileNotFoundError:
        global_setting = {}
    global_setting[key] = value
    base_folder = os.path.expanduser("~/pdfdeal")
    global_setting_file = os.path.join(base_folder, "global_setting.json")
    os.makedirs(base_folder, exist_ok=True)
    with open(global_setting_file, "w") as file:
        json.dump(global_setting, file)
    print("✅ Done!")


def delete_one_global_setting(key: str):
    """Delete one global setting"""
    global_setting = get_global_setting()
    if key in global_setting:
        del global_setting[key]
    base_folder = os.path.expanduser("~/pdfdeal")
    global_setting_file = os.path.join(base_folder, "global_setting.json")
    with open(global_setting_file, "w") as file:
        json.dump(global_setting, file)
    print("✅ Done!")


def get_global_setting() -> dict:
    """Get the global setting for pdfdeal"""
    base_folder = os.path.expanduser("~/pdfdeal")
    global_setting_file = os.path.join(base_folder, "global_setting.json")
    if not os.path.exists(global_setting_file):
        raise FileNotFoundError(
            "The global setting file does not exist, please set the global setting first."
        )
    with open(global_setting_file, "r") as file:
        global_setting = json.load(file)
    return global_setting


def set_local_setting(folder_name: str = None):
    """Set the local setting for pdfdeal"""
    from .config import init_local_config

    base_folder = os.path.expanduser("~/pdfdeal")
    folder_name, local_config = init_local_config(folder_name=folder_name)
    local_setting_file = os.path.join(base_folder, folder_name, "local_setting.json")
    with open(local_setting_file, "w") as file:
        json.dump(local_config, file)
    print("✅ Done!")
